//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.

///@file
/// Implements members of the MachineInfo class which wraps a bunch of
/// OS queries associated with machine names, etc.


#include <cxxtls/machine.h>
#ifdef _MSC_VER
#include <io.h>
#include <portable_io.h>
static std::string hostname_cache;
static std::string uname_cache;
static bool cached=false;
#else
#include <sys/utsname.h>
#endif

namespace cxxtls
{

MachineInfo
MachineInfo::get()
{
  MachineInfo rv;

#if _MSC_VER

 if(!cached)
 {
   cached=true;

   FILE *hostname_command=_popen("hostname", "r");

   char buffer[2000];

   if(hostname_command)
   {
      strcpy(buffer, "unknown\r\n");

      fgets(buffer, 2000, hostname_command);

      hostname_cache = buffer;

      _pclose(hostname_command);

   }

   FILE *uname_command=_popen("cmd /c ver", "r");

   if(uname_command)
   {
      strcpy(buffer, "Windows NT Version 4\r\n");

      fgets(buffer, 2000, uname_command);  // first line is blank!
      fgets(buffer, 2000, uname_command);

      char osname[2000];

      sscanf(buffer, "%s", osname);

      uname_cache = osname;

      _pclose(uname_command);

   }

 }

 rv.operating_system_  = uname_cache;
 rv.hostname_          = hostname_cache;


 return rv;

#else
  struct utsname info;

  if(uname(&info) < 0)
    return rv;

  rv.operating_system_ = info.sysname;
  rv.hostname_         = info.nodename;

  return rv;
#endif


}

} // namespace cxxtls
